@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 Testing using the Test Client
 
@@ -20,7 +19,9 @@ testing against the contexts and templates produced by a view,
 rather than the HTML rendered to the end-user.
 
 """
-from __future__ import unicode_literals
+import itertools
+import tempfile
+from unittest import mock
 
 from django.contrib.auth.models import User
 from django.core import mail
@@ -30,7 +31,7 @@ from django.test import (
 )
 from django.urls import reverse_lazy
 
-from .views import get_view, post_view, trace_view
+from .views import TwoArgException, get_view, post_view, trace_view
 
 
 @override_settings(ROOT_URLCONF='test_client.urls')
@@ -53,6 +54,19 @@ class ClientTest(TestCase):
         self.assertEqual(response.context['var'], '\xf2')
         self.assertEqual(response.templates[0].name, 'GET Template')
 
+    def test_query_string_encoding(self):
+        # WSGI requires latin-1 encoded strings.
+        response = self.client.get('/get_view/?var=1\ufffd')
+        self.assertEqual(response.context['var'], '1\ufffd')
+
+    def test_get_data_none(self):
+        msg = (
+            'Cannot encode None in a query string. Did you mean to pass an '
+            'empty string or omit the value?'
+        )
+        with self.assertRaisesMessage(TypeError, msg):
+            self.client.get('/get_view/', {'value': None})
+
     def test_get_post_view(self):
         "GET a view that normally expects POSTs"
         response = self.client.get('/post_view/', {})
@@ -86,6 +100,53 @@ class ClientTest(TestCase):
         self.assertEqual(response.templates[0].name, 'POST Template')
         self.assertContains(response, 'Data received')
 
+    def test_post_data_none(self):
+        msg = (
+            'Cannot encode None as POST data. Did you mean to pass an empty '
+            'string or omit the value?'
+        )
+        with self.assertRaisesMessage(TypeError, msg):
+            self.client.post('/post_view/', {'value': None})
+
+    def test_json_serialization(self):
+        """The test client serializes JSON data."""
+        methods = ('post', 'put', 'patch', 'delete')
+        tests = (
+            ({'value': 37}, {'value': 37}),
+            ([37, True], [37, True]),
+            ((37, False), [37, False]),
+        )
+        for method in methods:
+            with self.subTest(method=method):
+                for data, expected in tests:
+                    with self.subTest(data):
+                        client_method = getattr(self.client, method)
+                        method_name = method.upper()
+                        response = client_method('/json_view/', data, content_type='application/json')
+                        self.assertEqual(response.status_code, 200)
+                        self.assertEqual(response.context['data'], expected)
+                        self.assertContains(response, 'Viewing %s page.' % method_name)
+
+    def test_json_encoder_argument(self):
+        """The test Client accepts a json_encoder."""
+        mock_encoder = mock.MagicMock()
+        mock_encoding = mock.MagicMock()
+        mock_encoder.return_value = mock_encoding
+        mock_encoding.encode.return_value = '{"value": 37}'
+
+        client = self.client_class(json_encoder=mock_encoder)
+        # Vendored tree JSON content types are accepted.
+        client.post('/json_view/', {'value': 37}, content_type='application/vnd.api+json')
+        self.assertTrue(mock_encoder.called)
+        self.assertTrue(mock_encoding.encode.called)
+
+    def test_put(self):
+        response = self.client.put('/put_view/', {'foo': 'bar'})
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.templates[0].name, 'PUT Template')
+        self.assertEqual(response.context['data'], "{'foo': 'bar'}")
+        self.assertEqual(response.context['Content-Length'], '14')
+
     def test_trace(self):
         """TRACE a view"""
         response = self.client.trace('/trace_view/')
@@ -101,9 +162,8 @@ class ClientTest(TestCase):
 
     def test_response_attached_request(self):
         """
-        Check that the returned response has a ``request`` attribute with the
-        originating environ dict and a ``wsgi_request`` with the originating
-        ``WSGIRequest`` instance.
+        The returned response has a ``request`` attribute with the originating
+        environ dict and a ``wsgi_request`` with the originating WSGIRequest.
         """
         response = self.client.get("/header_view/")
 
@@ -141,8 +201,7 @@ class ClientTest(TestCase):
         test_doc = """<?xml version="1.0" encoding="utf-8"?>
         <library><book><title>Blink</title><author>Malcolm Gladwell</author></book></library>
         """
-        response = self.client.post("/raw_post_view/", test_doc,
-                                    content_type="text/xml")
+        response = self.client.post('/raw_post_view/', test_doc, content_type='text/xml')
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response.templates[0].name, "Book template")
         self.assertEqual(response.content, b"Blink - Malcolm Gladwell")
@@ -162,34 +221,34 @@ class ClientTest(TestCase):
     def test_redirect(self):
         "GET a URL that redirects elsewhere"
         response = self.client.get('/redirect_view/')
-        # Check that the response was a 302 (redirect)
         self.assertRedirects(response, '/get_view/')
 
     def test_redirect_with_query(self):
         "GET a URL that redirects with given GET parameters"
         response = self.client.get('/redirect_view/', {'var': 'value'})
-
-        # Check if parameters are intact
         self.assertRedirects(response, '/get_view/?var=value')
 
+    def test_redirect_with_query_ordering(self):
+        """assertRedirects() ignores the order of query string parameters."""
+        response = self.client.get('/redirect_view/', {'var': 'value', 'foo': 'bar'})
+        self.assertRedirects(response, '/get_view/?var=value&foo=bar')
+        self.assertRedirects(response, '/get_view/?foo=bar&var=value')
+
     def test_permanent_redirect(self):
         "GET a URL that redirects permanently elsewhere"
         response = self.client.get('/permanent_redirect_view/')
-        # Check that the response was a 301 (permanent redirect)
         self.assertRedirects(response, '/get_view/', status_code=301)
 
     def test_temporary_redirect(self):
         "GET a URL that does a non-permanent redirect"
         response = self.client.get('/temporary_redirect_view/')
-        # Check that the response was a 302 (non-permanent redirect)
         self.assertRedirects(response, '/get_view/', status_code=302)
 
     def test_redirect_to_strange_location(self):
         "GET a URL that redirects to a non-200 page"
         response = self.client.get('/double_redirect_view/')
-
-        # Check that the response was a 302, and that
-        # the attempt to get the redirection location returned 301 when retrieved
+        # The response was a 302, and that the attempt to get the redirection
+        # location returned 301 when retrieved
         self.assertRedirects(response, '/permanent_redirect_view/', target_status_code=301)
 
     def test_follow_redirect(self):
@@ -210,6 +269,39 @@ class ClientTest(TestCase):
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response.request['PATH_INFO'], '/accounts/login/')
 
+    def test_follow_307_and_308_redirect(self):
+        """
+        A 307 or 308 redirect preserves the request method after the redirect.
+        """
+        methods = ('get', 'post', 'head', 'options', 'put', 'patch', 'delete', 'trace')
+        codes = (307, 308)
+        for method, code in itertools.product(methods, codes):
+            with self.subTest(method=method, code=code):
+                req_method = getattr(self.client, method)
+                response = req_method('/redirect_view_%s/' % code, data={'value': 'test'}, follow=True)
+                self.assertEqual(response.status_code, 200)
+                self.assertEqual(response.request['PATH_INFO'], '/post_view/')
+                self.assertEqual(response.request['REQUEST_METHOD'], method.upper())
+
+    def test_follow_307_and_308_preserves_post_data(self):
+        for code in (307, 308):
+            with self.subTest(code=code):
+                response = self.client.post('/redirect_view_%s/' % code, data={'value': 'test'}, follow=True)
+                self.assertContains(response, 'test is the value')
+
+    def test_follow_307_and_308_preserves_put_body(self):
+        for code in (307, 308):
+            with self.subTest(code=code):
+                response = self.client.put('/redirect_view_%s/?to=/put_view/' % code, data='a=b', follow=True)
+                self.assertContains(response, 'a=b is the body')
+
+    def test_follow_307_and_308_preserves_get_params(self):
+        data = {'var': 30, 'to': '/get_view/'}
+        for code in (307, 308):
+            with self.subTest(code=code):
+                response = self.client.get('/redirect_view_%s/' % code, data=data, follow=True)
+                self.assertContains(response, '30 is the value')
+
     def test_redirect_http(self):
         "GET a URL that redirects to an http URI"
         response = self.client.get('/http_redirect_view/', follow=True)
@@ -223,8 +315,6 @@ class ClientTest(TestCase):
     def test_notfound_response(self):
         "GET a URL that responds as '404:Not Found'"
         response = self.client.get('/bad_view/')
-
-        # Check that the response was a 404, and that the content contains MAGIC
         self.assertContains(response, 'MAGIC', status_code=404)
 
     def test_valid_form(self):
@@ -249,7 +339,7 @@ class ClientTest(TestCase):
         response = self.client.get('/form_view/', data=hints)
         self.assertEqual(response.status_code, 200)
         self.assertTemplateUsed(response, "Form GET Template")
-        # Check that the multi-value data has been rolled out ok
+        # The multi-value data has been rolled out ok
         self.assertContains(response, 'Select a valid choice.', 0)
 
     def test_incomplete_data_form(self):
@@ -334,14 +424,14 @@ class ClientTest(TestCase):
         "GET an invalid URL"
         response = self.client.get('/unknown_view/')
 
-        # Check that the response was a 404
+        # The response was a 404
         self.assertEqual(response.status_code, 404)
 
     def test_url_parameters(self):
         "Make sure that URL ;-parameters are not stripped."
         response = self.client.get('/unknown_view/;some-parameter')
 
-        # Check that the path in the response includes it (ignore that it's a 404)
+        # The path in the response includes it (ignore that it's a 404)
         self.assertEqual(response.request['PATH_INFO'], '/unknown_view/;some-parameter')
 
     def test_view_with_login(self):
@@ -556,6 +646,17 @@ class ClientTest(TestCase):
         self.assertEqual(response.context['user'].username, 'testclient')
         self.assertEqual(self.u1.backend, 'django.contrib.auth.backends.ModelBackend')
 
+    @override_settings(AUTHENTICATION_BACKENDS=[
+        'test_client.auth_backends.BackendWithoutGetUserMethod',
+        'django.contrib.auth.backends.ModelBackend',
+    ])
+    def test_force_login_with_backend_missing_get_user(self):
+        """
+        force_login() skips auth backends without a get_user() method.
+        """
+        self.client.force_login(self.u1)
+        self.assertEqual(self.u1.backend, 'django.contrib.auth.backends.ModelBackend')
+
     @override_settings(SESSION_ENGINE="django.contrib.sessions.backends.signed_cookies")
     def test_logout_cookie_sessions(self):
         self.test_logout()
@@ -613,6 +714,21 @@ class ClientTest(TestCase):
         response = self.client.get('/django_project_redirect/')
         self.assertRedirects(response, 'https://www.djangoproject.com/', fetch_redirect_response=False)
 
+    def test_external_redirect_with_fetch_error_msg(self):
+        """
+        assertRedirects without fetch_redirect_response=False raises
+        a relevant ValueError rather than a non-descript AssertionError.
+        """
+        response = self.client.get('/django_project_redirect/')
+        msg = (
+            "The test client is unable to fetch remote URLs (got "
+            "https://www.djangoproject.com/). If the host is served by Django, "
+            "add 'www.djangoproject.com' to ALLOWED_HOSTS. "
+            "Otherwise, use assertRedirects(..., fetch_redirect_response=False)."
+        )
+        with self.assertRaisesMessage(ValueError, msg):
+            self.assertRedirects(response, 'https://www.djangoproject.com/')
+
     def test_session_modifying_view(self):
         "Request a page that modifies the session"
         # Session value isn't set initially
@@ -620,7 +736,7 @@ class ClientTest(TestCase):
             self.client.session['tobacconist']
 
         self.client.post('/session_view/')
-        # Check that the session was modified
+        # The session was modified
         self.assertEqual(self.client.session['tobacconist'], 'hovercraft')
 
     @override_settings(
@@ -644,8 +760,7 @@ class ClientTest(TestCase):
             self.client.get("/broken_view/")
 
     def test_mail_sending(self):
-        "Test that mail is redirected to a dummy outbox during test setup"
-
+        "Mail is redirected to a dummy outbox during test setup"
         response = self.client.get('/mail_sending_view/')
         self.assertEqual(response.status_code, 200)
 
@@ -657,7 +772,7 @@ class ClientTest(TestCase):
         self.assertEqual(mail.outbox[0].to[1], 'second@example.com')
 
     def test_reverse_lazy_decodes(self):
-        "Ensure reverse_lazy works in the test client"
+        "reverse_lazy() works in the test client"
         data = {'var': 'data'}
         response = self.client.get(reverse_lazy('get_view'), data)
 
@@ -673,8 +788,7 @@ class ClientTest(TestCase):
         self.assertRedirects(response, '/accounts/login/')
 
     def test_mass_mail_sending(self):
-        "Test that mass mail is redirected to a dummy outbox during test setup"
-
+        "Mass mail is redirected to a dummy outbox during test setup"
         response = self.client.get('/mass_mail_sending_view/')
         self.assertEqual(response.status_code, 200)
 
@@ -699,6 +813,22 @@ class ClientTest(TestCase):
         with self.assertRaisesMessage(Exception, 'exception message'):
             self.client.get('/nesting_exception_view/')
 
+    def test_response_raises_multi_arg_exception(self):
+        """A request may raise an exception with more than one required arg."""
+        with self.assertRaises(TwoArgException) as cm:
+            self.client.get('/two_arg_exception/')
+        self.assertEqual(cm.exception.args, ('one', 'two'))
+
+    def test_uploading_temp_file(self):
+        with tempfile.TemporaryFile() as test_file:
+            response = self.client.post('/upload_view/', data={'temp_file': test_file})
+        self.assertEqual(response.content, b'temp_file')
+
+    def test_uploading_named_temp_file(self):
+        test_file = tempfile.NamedTemporaryFile()
+        response = self.client.post('/upload_view/', data={'named_temp_file': test_file})
+        self.assertEqual(response.content, b'named_temp_file')
+
 
 @override_settings(
     MIDDLEWARE=['django.middleware.csrf.CsrfViewMiddleware'],
@@ -709,11 +839,9 @@ class CSRFEnabledClientTests(SimpleTestCase):
     def test_csrf_enabled_client(self):
         "A client can be instantiated with CSRF checks enabled"
         csrf_client = Client(enforce_csrf_checks=True)
-
         # The normal client allows the post
         response = self.client.post('/post_view/', {})
         self.assertEqual(response.status_code, 200)
-
         # The CSRF-enabled client rejects it
         response = csrf_client.post('/post_view/', {})
         self.assertEqual(response.status_code, 403)
@@ -750,9 +878,7 @@ class RequestFactoryTest(SimpleTestCase):
         ('options', _generic_view),
         ('trace', trace_view),
     )
-
-    def setUp(self):
-        self.request_factory = RequestFactory()
+    request_factory = RequestFactory()
 
     def test_request_factory(self):
         """The request factory implements all the HTTP/1.1 methods."""
@@ -760,7 +886,6 @@ class RequestFactoryTest(SimpleTestCase):
             method = getattr(self.request_factory, method_name)
             request = method('/somewhere/')
             response = view(request)
-
             self.assertEqual(response.status_code, 200)
 
     def test_get_request_from_factory(self):
@@ -769,7 +894,6 @@ class RequestFactoryTest(SimpleTestCase):
         """
         request = self.request_factory.get('/somewhere/')
         response = get_view(request)
-
         self.assertContains(response, 'This is a test')
 
     def test_trace_request_from_factory(self):
@@ -779,5 +903,4 @@ class RequestFactoryTest(SimpleTestCase):
         response = trace_view(request)
         protocol = request.META["SERVER_PROTOCOL"]
         echoed_request_line = "TRACE {} {}".format(url_path, protocol)
-
         self.assertContains(response, echoed_request_line)
